@@ -45,7 +45,7 @@ EpiInvert_plot <- function(x, what = "all",date_start="1000-01-01",date_end="300
       ggplot2::geom_line() + ggplot2::ylim(0, 1.02 * max(d$Rt)) + 
       ggplot2::scale_x_date(date_labels = "%Y-%m-%d") + 
       theme_graphs +
-      ggplot2::geom_ribbon(ggplot2::aes(ymin = Rt -  Rt_CI95, ymax = Rt + Rt_CI95), fill = I(rgb(0.7,  0.7, 0.7))) + 
+      ggplot2::geom_ribbon(ggplot2::aes(ymin = Rt -  Rt_CI95, ymax = Rt + Rt_CI95), fill = I(rgb(0.7,  0.7, 0.7)),colour = NA) + 
       ggplot2::geom_line(ggplot2::aes(y = Rt)) + 
       ggplot2::geom_hline(yintercept = 1) + ggplot2::geom_text(data=data.frame(1,1),x = -Inf, 
                                                                y = Inf, label = "Rt", hjust = -0.5, vjust = 1.5,inherit.aes = F)
@@ -118,7 +118,7 @@ EpiInvert_plot <- function(x, what = "all",date_start="1000-01-01",date_end="300
     
     g5 <- ggplot2::ggplot(d, ggplot2::aes(x = date, y = Rt)) + 
       ggplot2::ylim(0, 1.2 * max(d$Rt)) + ggplot2::geom_line() + 
-      theme_graphs_no_xlab + ggplot2::geom_ribbon(ggplot2::aes(ymin = Rt - Rt_CI95, ymax = Rt + Rt_CI95), fill = I(rgb(0.7, 0.7, 0.7))) + 
+      theme_graphs_no_xlab + ggplot2::geom_ribbon(ggplot2::aes(ymin = Rt - Rt_CI95, ymax = Rt + Rt_CI95), fill = I(rgb(0.7, 0.7, 0.7)), colour = NA) + 
       ggplot2::geom_line(ggplot2::aes(y = Rt)) + 
       ggplot2::geom_hline(yintercept = 1) + 
       ggplot2::geom_text(data=data.frame(1,1),x = -Inf,  y = Inf, label = "Rt", hjust = -0.9, vjust = 1.5,inherit.aes = F)
@@ -146,6 +146,93 @@ EpiInvert_plot <- function(x, what = "all",date_start="1000-01-01",date_end="300
     grid::grid.draw(rbind(p1, p2, p3, p4, p5, p6, p7))
   }
   else {
-    stop("The drawing option is not recognized")
+    stop("The drawing option is not recognized.")
   }
-}
\ No newline at end of file
+}
+
+#' @title 
+#' \code{EpiInvertForecast_plot} plot the restored incidence forecast
+#'
+#' @param EpiInvert_results the list returned by the EpiInvert execution
+#'
+#' @param Forecast the list returned by the EpiInvertForecast execution
+#'
+#' @param what one of the following drawing options: 
+#' 
+#'  \itemize{
+#'  \item{CI50}{: use confidence interval for the restored incidence forecast with percentil 50.}
+#'  \item{CI75}{: use confidence interval for the restored incidence forecast with percentil 75. }
+#'  \item{CI90}{: use confidence interval for the restored incidence forecast with percentil 90. }
+#'  \item{CI95}{: use confidence interval for the restored incidence forecast with percentil 95. }
+#'  
+#' }
+#' 
+#' @return a plot with the last 28 days of the original and restored incidence curves and a 28-day 
+#' forecast of the same curves. It also includes a shaded area with an empiric confidence interval
+#' of the restored incidence forecast estimation
+#' 
+
+#' @export
+EpiInvertForecast_plot <- function(EpiInvert_results,Forecast,what = "CI95"){
+  
+  Do <- utils::tail(EpiInvert_results$dates,28)
+  Oo <- utils::tail(EpiInvert_results$i_original,28)
+  Ro <- utils::tail(EpiInvert_results$i_restored,28)
+  
+  Df <- Forecast$dates
+  Of <- Forecast$i_original_forecast
+  Rf <- Forecast$i_restored_forecast
+  Cf50 <- Forecast$i_restored_forecast_CI50
+  Cf75 <- Forecast$i_restored_forecast_CI75
+  Cf90 <- Forecast$i_restored_forecast_CI90
+  Cf95 <- Forecast$i_restored_forecast_CI95
+  
+  if (what == "CI50") {
+    CImin <- pmax(Rf-Cf50,0)
+    CImax <- Rf+Cf50
+  }
+  else if (what == "CI75") {
+    CImin <- pmax(Rf-Cf75,0)
+    CImax <- Rf+Cf75
+  }
+  else if (what == "CI90") {
+    CImin <- pmax(Rf-Cf90,0)
+    CImax <- Rf+Cf90
+  }
+  else if (what == "CI95") {
+    CImin <- pmax(Rf-Cf95,0)
+    CImax <- Rf+Cf95
+  }
+  else {
+    stop("The drawing option is not recognized. The options are \"CI50\", \"CI75\", \"CI90\" and \"CI95\" ")
+  }  
+   
+  #utils::globalVariables(c("date", "incid", "legend", "ymi", "yma"))
+  dfr <- data.frame(
+        date = c(as.Date(Do),as.Date(Do),as.Date(Df),as.Date(Df)),
+        incid = c(Oo,Ro,Of,Rf),
+        legend2 = c(rep("original incidence",28),rep("restored incidence",28),rep("forecast original incidence",28),rep("forecast restored incidence",28)),
+        ymi = c(Oo,Ro,Of,CImin),
+        yma = c(Oo,Ro,Of,CImax)
+  )
+  
+  date2 <- dfr$date
+  incid2 <- dfr$incid
+  legend <- dfr$legend
+  ymi2 <- dfr$ymi
+  yma2 <- dfr$yma
+  
+                  
+  g <- ggplot2::ggplot(dfr, ggplot2::aes(date2,incid2, col=legend))+ 
+       ggplot2::geom_ribbon(ggplot2::aes(ymin = ymi2, ymax = yma2), fill = I(rgb(0.9,  0.9, 0.9)), linetype = 0 )+ 
+       ggplot2::geom_line(size = 0.7)+
+       ggplot2::scale_color_manual(values = c("blue",  "green", "black", "red")) + 
+       ggplot2::scale_x_date(date_labels = "%Y-%m-%d") + 
+       ggplot2::theme_bw()+
+       ggplot2::theme(axis.title = ggplot2::element_blank() )+
+       ggplot2::theme(legend.position="bottom")
+  
+  p <- ggplot2::ggplotGrob(g)
+  grid::grid.draw(rbind(p))
+ 
+  }
\ No newline at end of file
